/******** <lalVerbatim file="FCCTonditionDataCV"> ******** 
Author: Wen, L. 
$Id: FCTConditionData.c,v 1.5 2001/08/27 20:05:30 lwen Exp $ 
********* </lalVerbatim> ********/ 

#include <FCTGeneral.h>
#include <FCTConditionData.h>

NRCSID( FCTCONDITIONDATAC, "$Id: FCTConditionData.c,v 1.5 2001/08/27 20:05:30 lwen Exp $");

void LALConditionData( 

		      LALStatus *status, 
		      LALSearchInput *inout, 
		      void *searchParams, 
		      LALMPIParams          *mpiParams
		      ) 
{ 
  
  FCTSearchParams *params;
  INT4                          i,j;
  INT4                          dmroIndex  = -1;
  INT4                          specIndex  = -1;
  INT4                          respIndex  = -1;
  REAL4                         *dummyI2    = NULL; 
  REAL4                        *dummyR4    = NULL;
  COMPLEX8                     *dummyC8    = NULL;
  INT4                           dataLength;
  /* DFT */
  COMPLEX8Vector* tmpIn; /*  timeseries data  */
  ComplexFFTPlan* plan;

  INITSTATUS( status, "LALConditionData", FCTCONDITIONDATAC ); 
  ATTATCHSTATUSPTR (status);

#ifdef FCT_SO_DEBUG  
  /*     lalDebugLevel=24; */
#endif

  mpiParams=NULL;
  params = (FCTSearchParams *) searchParams;

#ifdef FCT_SO_DEBUG
   fprintf(stdout," in dataCondition()\n");
   fflush( stdout );
#endif

   /*  manipulating input to fit in FCT input */
   /* copy input in  */

   if ( params->searchMaster )
  {
    /* return immediately */
    DETATCHSTATUSPTR (status);
    RETURN (status);
  } 
   else /*  if slave, copy data from the ilwd input data format */
  {

#ifdef FCT_SO_DEBUG  
     fprintf(stdout,"inout->numberSequences=%i\n",inout->numberSequences); 
     fflush( stdout );
     fprintf(stdout,"inout->sequences[0].name=%s\n",inout->sequences[0].name); 
     fflush( stdout );
#endif

    for ( i = 0; i < inout->numberSequences; ++i )
    {
      if (strstr( inout->sequences[i].name, INPUTNAME_IFODMRO))  
      {
        dmroIndex = i;

        dummyI2   = inout->sequences[i].data.real4;
      }
      else if (strstr( inout->sequences[i].name, INPUTNAME_SPECTRUM)) 
      {
        specIndex = i;
        dummyR4   = inout->sequences[i].data.real4;

      }
      else if (strstr( inout->sequences[i].name, INPUTNAME_RESPONSE)) 
      {
        respIndex = i;
        dummyC8   = inout->sequences[i].data.complex8;
      }
      else
        ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
    }/* end for */

    params->fctInParams->epoch.gpsSeconds =
      inout->sequences[dmroIndex].range.dTime.startSec;
    params->fctInParams->epoch.gpsNanoSeconds = 
      inout->sequences[dmroIndex].range.dTime.startNan;
    params->fctInParams->maxF=(INT4) (1.0/inout->sequences[dmroIndex].range.dTime.timeStepSize);
#ifdef FCT_SO_DEBUG  
    printf("epoch is %i,+%i, maxF=%i\n",params->fctInParams->epoch.gpsSeconds,params->fctInParams->epoch.gpsNanoSeconds,params->fctInParams->maxF);
#endif
    dataLength=params->fctInParams->dataLength;

#ifdef FCT_SO_DEBUG
   
    printf(" data length is %i\n",dataLength);
#endif

    if ( inout->sequences[dmroIndex].dimensions[0] < dataLength)
    {
      ABORT (status, CONDITIONDATA_EDATZ, CONDITIONDATA_MSGEDATZ); 
    }
    

    if (dmroIndex == -1)
    {
      ABORT (status, CONDITIONDATA_EINPUT, CONDITIONDATA_MSGEINPUT); 
    }


    tmpIn=NULL;
    LALCCreateVector(status->statusPtr, &tmpIn, dataLength);
    CHECKSTATUSPTR (status);
    

    /*  do DFT here , a simple one */
      
    for (j = 0; j <dataLength; ++j)
    {
      tmpIn->data[j].re = (REAL4) dummyI2[j];
      tmpIn->data[j].im = 0.0;

#ifdef FCT_SO_DEBUG
      printf("data are %g\n",(REAL4) dummyI2[j]);
#endif

    }
      
    /*  Plan for doing the FFT on  the tseries */

    plan=NULL;
    /*  equavilent to fwd FFT in FFTW  */
    LALEstimateInvComplexFFTPlan(status->statusPtr, &plan, tmpIn->length);
    CHECKSTATUSPTR (status);

    LALCOMPLEX8VectorFFT(status->statusPtr, params->fctInParams->fctInput, tmpIn, plan);
    CHECKSTATUSPTR (status);
    
    
#ifdef FCT_SO_DEBUG    
    for (j=0; j<dataLength; j++)
    {
      printf("fctInput are %g, %g\n",params->fctInParams->fctInput->data[j].re,params->fctInParams->fctInput->data[j].im);
    }
#endif  
    
    params->fctInParams->dataLength/=2;
    params->fctInParams->fctInput->length/=2;

    dataLength=params->fctInParams->dataLength;

    /*  temporary way to deal with response and noise power  */ 
    /* zero-out coefficients with  frequency < 40 out for  max f= 1024 */
    for (j =0; j < 5*dataLength/128; ++j)
    {
      tmpIn->data[j].re = 0.;
      tmpIn->data[j].im = 0.;
    } /*  suppose to be 1/noiseP*Response */
 
    /* 1/N  normalization for FFT  */
    for (j = 5*dataLength/128; j <dataLength; ++j)
    {
      tmpIn->data[j].re = 1.0/dataLength/2;
      tmpIn->data[j].im = 1.0/dataLength/2;
    } /*  suppose to be 1/noiseP*Response */

    for (j = 0; j <dataLength; ++j)
    {
      params->fctInParams->fctInput->data[j].re *=tmpIn->data[j].re;
      params->fctInParams->fctInput->data[j].im *=tmpIn->data[j].im;
    }
    
    
    LALCDestroyVector(status->statusPtr, &tmpIn);
    CHECKSTATUSPTR (status);
      
    LALDestroyComplexFFTPlan(status->statusPtr, &plan);
    CHECKSTATUSPTR (status);
    
  }/*  end if slave  */

#ifdef FCT_SO_DEBUG  
   fprintf(stdout," out of ConditionData()\n");
   fflush( stdout );
#endif

  DETATCHSTATUSPTR (status);
  RETURN( status ); 
} 


