/*----------------------------------------------------------------------- 
 * 
 * File Name: ConditionDataTmplt.c
 *
 * Author: Brown, D. A. and Creighton T. D.
 * 
 * Revision: $Id: ConditionDataTmplt.c,v 1.5 2001/05/17 18:58:48 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ConditionData.h"

NRCSID( CONDITIONDATATMPLTC, PKG "$Id: ConditionDataTmplt.c,v 1.5 2001/05/17 18:58:48 duncan Exp $" );

void
InspiralTmpltsFromILWD (
    LALStatus                   *status,
    InspiralSearchParams        *params,
    multiDimData                *mddTmplt
                       )
{
  UINT4                         i;
  UINT4                         tmpltIndex  = 0;
  UINT4                         numTmplts   = 0;
  REAL8                         m1, m2;
  InspiralTemplate             *here        = NULL;


  INITSTATUS( status, "InspiralTmpltsFromILWD", CONDITIONDATATMPLTC );
  ATTATCHSTATUSPTR( status );


  ASSERT( params, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );
  ASSERT( mddTmplt, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );


  /*
   *
   * sanity check on the ilwd template bank input
   *
   */


  if ( mddTmplt->numberDimensions != 2 )
  {
    ABORT( status, CONDITIONDATAH_EMALF, CONDITIONDATAH_MSGEMALF );
  }
  if ( mddTmplt->dimensions[0] != 2 )
  {
    ABORT( status, CONDITIONDATAH_EMALF, CONDITIONDATAH_MSGEMALF );
  }
  if ( mddTmplt->dimensions[1] < 1 )
  {
    ABORT( status, CONDITIONDATAH_EMALF, CONDITIONDATAH_MSGEMALF );
  }


  /*
   *
   * get the template bank from the input
   *
   */


  numTmplts = mddTmplt->dimensions[1];

  /* the first template is the head of the linked list */
  here = params->bankHead = (InspiralTemplate *)
    LALCalloc( 1, sizeof( InspiralTemplate ) );
  ASSERT( here, status, 
      CONDITIONDATAH_EALOC, CONDITIONDATAH_MSGEALOC );

  /* get the mass parameters */
  m1 = mddTmplt->data.real8[0];
  m2 = mddTmplt->data.real8[1];

  /* set the template parameters */
  here->number = tmpltIndex++;
  here->mass1 = m1;
  here->mass2 = m2;
  here->totalMass = m1 + m2;
  here->mu = m1*m2/here->totalMass;
  here->eta = here->mu/here->totalMass;

  LALI4CreateVector( status->statusPtr, 
      &(here->segmentIdVec), params->initParams->numSegments );
  CHECKSTATUSPTR( status );
  memset( here->segmentIdVec->data, 0, 
      here->segmentIdVec->length * sizeof(INT4) );
  
  /* do the rest of the bank */
  for ( i = 2; i < numTmplts * 2; i = i + 2 )
  {
    /* allocate the next member of the list */
    here->next = (InspiralTemplate *)
      LALCalloc( 1, sizeof( InspiralTemplate ) );
    here = here->next;
    ASSERT( here, status, 
        CONDITIONDATAH_EALOC, CONDITIONDATAH_MSGEALOC );

    /* get the mass parameters */
    m1 = mddTmplt->data.real8[i];
    m2 = mddTmplt->data.real8[i+1];

    /* set the template parameters */
    here->number = tmpltIndex++;
    here->mass1 = m1;
    here->mass2 = m2;
    here->totalMass = m1 + m2;
    here->mu = m1*m2/here->totalMass;
    here->eta = here->mu/here->totalMass;

    LALI4CreateVector( status->statusPtr, 
        &(here->segmentIdVec), params->initParams->numSegments );
    CHECKSTATUSPTR( status );
    memset( here->segmentIdVec->data, 0, 
        here->segmentIdVec->length * sizeof(INT4) );
  }


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
