/*----------------------------------------------------------------------- 
 * 
 * File Name: FinalizeSearch.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: FinalizeSearch.c,v 1.8 2001/05/30 22:10:28 duncan Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "FinalizeSearch.h"

NRCSID( FINALIZESEARCHC, PKG "$Id: FinalizeSearch.c,v 1.8 2001/05/30 22:10:28 duncan Exp $" );

void
LALFinalizeSearch(
    LALStatus             *status,
    void                 **searchParams
    )
{
  InspiralSearchParams *params;

  INITSTATUS( status, "LALFinalizeSearch", FINALIZESEARCHC );
  ATTATCHSTATUSPTR( status );

  ASSERT( *searchParams, status,
          FINALIZESEARCH_ENULL, FINALIZESEARCH_MSGENULL );

  params = (InspiralSearchParams *) *searchParams;

  if ( params->searchMaster )
  {
    /* free the linked list of template */
    LALFindChirpDestroyInspiralBank( status->statusPtr, 
        &(params->bankHead) );
    CHECKSTATUSPTR( status );

    /* destroy linked list of template nodes */
    while ( params->tmpltHead )
    {
      LALFindChirpDestroyTmpltNode( status->statusPtr, &(params->tmpltHead) );
      CHECKSTATUSPTR( status );
    }

    LALFree( params->numSlaves );
  }
  else /* searchSlave */
  {
    /* finalize chisq veto */
    LALFindChirpChisqVetoFinalize( status->statusPtr, 
        params->filterParams->chisqParams, params->numChisqBins );
    CHECKSTATUSPTR( status );

    /* finalize data conditioning function */
    LALFindChirpSPDataFinalize( status->statusPtr, &(params->dataParams) );
    CHECKSTATUSPTR( status );

    /* destroy search input structure */
    LALDestroyFindChirpInput( status->statusPtr, &(params->filterInput) );
    CHECKSTATUSPTR( status );

    /* finalize template generation */
    LALFindChirpSPTemplateFinalize( status->statusPtr, &(params->tmpltParams) );
    CHECKSTATUSPTR( status );

    /* finalize filter */
    LALFindChirpFilterFinalize( status->statusPtr, &(params->filterParams) );
    CHECKSTATUSPTR( status );

    /* free conditioned segments */
    LALDestroyFindChirpSegmentVector( status->statusPtr, &(params->fcSegVec) );
    CHECKSTATUSPTR( status );

    /* free thresholds */
    LALFree( params->rhosqThreshVec );
    LALFree( params->chisqThreshVec );
  }

  /* free findchirp initialisation parameters */
  LALFree( params->initParams );

  /* free any memory used for debugging */
  if ( params->loudestEvent )
  {
#ifdef INSPIRAL_SO_DEBUG
    fprintf( stderr, "freeing memory for loudest event at %p\n",
        params->loudestEvent );
#endif
    LALFree( params->loudestEvent );
  }

  /* free searchParams */
  LALFree( *searchParams );
  *searchParams = NULL;

  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}

