/********************************** <lalVerbatim file="PowerInitSearchCV">
Author: Brady, P and Brown, D
$Id: BuildOutput.c,v 1.12 2001/09/03 22:07:49 patrick Exp $
**************************************************** </lalVerbatim> */

#include <lal/LALStdlib.h>
#include <lal/LALDatatypes.h>
#include <lal/ExcessPower.h>
#include <lal/TFTransform.h>
#include <lal/Comm.h>
#include <lal/AVFactories.h>
#include "ApplySearch.h"
#include "EPData.h"

NRCSID( BUILDPOWEROUTPUTC, "power $Id: BuildOutput.c,v 1.12 2001/09/03 22:07:49 patrick Exp $");

void
BuildPowerOutPut (
    LALStatus          *status,
    LALSearchOutput    *output,
    TFTile             *tfTile,
    PowerOutPutParams  *params
                    )
{
  TFTile               *event;
  outPut               *outputArray;
  dataBase             *prev;
  dataBase             *dbEntry;
  INT4                 i;
  INT8                 dummyNS=0;

  INITSTATUS (status, "BuildPowerOutPut", BUILDPOWEROUTPUTC);

  output->numOutput = params->numEvents;

  /*
   *
   * print the list events as recieved by the master
   *
   */


#ifdef POWER_SO_DBGLVL3
  {
    INT4 j=0;
    event = tfTile;

    do
    {

      fprintf( stdout, "build output: numEvents %d\n", params->numEvents);
      fflush( stdout );
      fprintf( stdout, "build output: fstart = %d\n", event-> fstart);
      fprintf( stdout, "build output: fend = %d\n", event-> fend);
      fprintf( stdout, "build output: tstart id = %d\n", event-> tstart);
      fprintf( stdout, "build output: tend = %d\n", event-> tend);
      fprintf( stdout, "build output: deltaT = %e\n", event-> deltaT);
      fprintf( stdout, "build output: whichPlane = %d\n", event-> whichPlane);
      fprintf( stdout, "build output: snrsq = %e\n", event->excessPower );
      fprintf( stdout, "build output: chisq = %e\n", event->alpha );
      fprintf( stdout, "build output: sigma = %e\n", event->weight );
      fflush( stdout );

      event = event->nextTile;
      j++;

    }while( j<params->numEvents);
  }
#endif


  /*
   *
   * create memory for the outPut array
   *
   */

  outputArray = output->result = (outPut *) LALMalloc( params->numEvents * sizeof(outPut) );


  /*
   *
   * fill the array of output
   *
   */


  for ( i = 0, event = tfTile; (i < params->numEvents) && event; ++i, event = event->nextTile )
  {
    /* this is the same for every output */
    outputArray[i].search         = burst;
    outputArray[i].significant    = 1;
    outputArray[i].states         = NULL;
    outputArray[i].optional       = NULL;

    /* May not be correct?  Does this need to be non-zero */
    outputArray[i].indexNumber = 0;

    /* 
     *
     * alloc memory for the database linked list
     *
     */

    /* entry: start_time */
    dummyNS = params->tstart + (INT8) (1e9 * tfTile->tstart * tfTile->deltaT);
    
    dbEntry = outputArray[i].results = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = NULL;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "start_time" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = (INT4) (dummyNS/1000000000L);

    prev = dbEntry;

    /* entry: start_time_ns */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "start_time_ns" );

    dbEntry->type       = int_4s;
    dbEntry->numberRows = 1;
    dbEntry->rows.int4s = (INT4 *) LALMalloc( sizeof(INT4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.int4s) = (INT4) (dummyNS%1000000000L);

    prev = dbEntry;

     /* entry: duration */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "duration" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4) (event->tend - event->tstart) * event->deltaT;

    prev = dbEntry;

    
    /* entry: central frequency */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "central_freq" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = params->flow + 
      (REAL4) (event->fstart + event->fend) / (2.0 * event->deltaT);

    prev = dbEntry;

    
    /* entry: bandwidth  */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "bandwidth" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = (REAL4) (event->fend - event->fstart) / (event->deltaT);

    prev = dbEntry;


    /* entry: amplitude must be non-null, but needs fixing */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "amplitude" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = event->excessPower;

    prev = dbEntry;


    /* entry: excess power */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev;

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "snr" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = event->excessPower;

    prev = dbEntry;


    /* entry: confidence */
    dbEntry = prev->next = (dataBase *) LALMalloc( sizeof(dataBase) );
    dbEntry->previous = prev; 

    sprintf( dbEntry->tableName, "sngl_burst" );
    sprintf( dbEntry->columnName, "confidence" );

    dbEntry->type       = real_4;
    dbEntry->numberRows = 1;
    dbEntry->rows.real4 = (REAL4 *) LALMalloc( sizeof(REAL4) );
    dbEntry->rowDimensions = NULL;
    *(dbEntry->rows.real4) = event->alpha;

    dbEntry->next = NULL;
  }


  RETURN (status);
}
