/*----------------------------------------------------------------------- 
 * 
 * File Name: ConditionData.c
 *
 * Author: Brown, D. A.
 * 
 * Revision: $Id: ConditionData.c,v 1.3 2001/08/27 17:23:23 patrick Exp $
 * 
 *-----------------------------------------------------------------------
 */

#include "ConditionData.h"

NRCSID( CONDITIONDATAC, PKG "$Id: ConditionData.c,v 1.3 2001/08/27 17:23:23 patrick Exp $" );


void
LALConditionData(
    LALStatus             *status,
    LALSearchInput        *inout,
    void                  *searchParams,
    LALMPIParams          *mpiParams
                )
{
  UINT4                 j;
  REAL4Vector           dummyVec;
  SimpleSearchParams   *params;

  INITSTATUS( status, "LALConditionData", CONDITIONDATAC );
  ATTATCHSTATUSPTR( status );


  /*
   *
   * check and cast parameter structure
   *
   */


  if ( ! searchParams )
  {
    ABORT( status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );
  }
  params = (SimpleSearchParams *) searchParams;


  /*
   *
   * check the input
   *
   */
  

  ASSERT( inout, status, CONDITIONDATAH_ENULL, CONDITIONDATAH_MSGENULL );

  /* check that there is only one input multiDimData sequence */
  if ( inout->numberSequences != 1 )
  {
    ABORT( status, CONDITIONDATAH_EINPT, CONDITIONDATAH_MSGEINPT );
  }

  /* check that the multiDimData sequence contains data of type REAL4 */
  if ( inout->sequences[0].type != real_4 )
  {
    ABORT( status, CONDITIONDATAH_EINPT, CONDITIONDATAH_MSGEINPT );
  }

  /* check that the data is of the expected dimensions */
  if ( inout->sequences[0].numberDimensions != 1 )
  {
    ABORT( status, CONDITIONDATAH_EINPT, CONDITIONDATAH_MSGEINPT );
  }
  if ( inout->sequences[0].dimensions[0] != params->numPoints )
  {
    ABORT( status, CONDITIONDATAH_EINPT, CONDITIONDATAH_MSGEINPT );
  }


  /*
   *
   * copy the input data from the multiDimData to the LAL REAL4Vector
   *
   */


  dummyVec.length = inout->sequences[0].dimensions[0];
  dummyVec.data   = inout->sequences[0].data.real4;


  /*
   *
   * condition the input data
   * 
   */
  

  /* we now have the input data in a LAL structure that we can pass to      */
  /* other LAL functions and perform and search-specific data conditioning  */
  /* not done in the dataconditioningAPI                                    */

  /* in this trivial example, we will just flip the sign of the input data  */
  /* see the inspiral or power shared objects for more complicated examples */

  /* flip the sign of the input data and store */
  for ( j = 0; j < params->dataVec->length; ++j )
  {
    params->dataVec->data[j] = -1.0 * dummyVec.data[j];
  }


  /* normal exit */
  DETATCHSTATUSPTR( status );
  RETURN( status );
}
