/******************************* <lalVerbatim file="StochasticBarInitSearchHV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticBarInitSearch.h,v 1.5 2001/09/02 05:55:11 heng Exp $
**************************************************** </lalVerbatim> */

/******************************* <lalLaTeX file="StochasticBarInitSearchH">
\section{Header \texttt{StochasticBarInitSearch.h}}
\label{s:StochasticBarInitSearch.h}

Header file for ...

\subsection*{Synopsis}
\begin{verbatim}
#include "StochasticBarInitSearch.h"
\end{verbatim}

\noindent
This header ...

\subsection*{Error conditions}
\input{StochasticBarInitSearchHErrTab}

\subsection*{Structures}

None.

*************************************************** </lalLaTeX> */

#ifndef _STOCHASTICINITSEARCH_H
#define _STOCHASTICINITSEARCH_H

#include <lal/LALStdlib.h>
#include "LALWrapperInterface.h"
#include "StochasticBar.h"

#ifdef  __cplusplus
extern "C" {
#endif

NRCSID (STOCHASTICINITSEARCHH, "$Id: StochasticBarInitSearch.h,v 1.5 2001/09/02 05:55:11 heng Exp $");

/******** <lalErrTable file="StochasticBarInitSearchHErrTab"> ********/
#define STOCHASTICINITSEARCHH_ENULL       1
#define STOCHASTICINITSEARCHH_ENNUL       2
#define STOCHASTICINITSEARCHH_EALOC       3
#define STOCHASTICINITSEARCHH_EARGS       4
#define STOCHASTICINITSEARCHH_EARG0       5
#define STOCHASTICINITSEARCHH_ENSEG       6
#define STOCHASTICINITSEARCHH_ENPTS       7
#define STOCHASTICINITSEARCHH_ENFRQ       8
#define STOCHASTICINITSEARCHH_EF0         9
#define STOCHASTICINITSEARCHH_EDELF       10
#define STOCHASTICINITSEARCHH_ESTID       11
#define STOCHASTICINITSEARCHH_ESIZE       12
#define STOCHASTICINITSEARCHH_EFRES       13
#define STOCHASTICINITSEARCHH_EFRNG       14

#define STOCHASTICINITSEARCHH_MSGENULL    "Null pointer"
#define STOCHASTICINITSEARCHH_MSGENNUL    "Non-null pointer"
#define STOCHASTICINITSEARCHH_MSGEALOC    "Memory allocation error"
#define STOCHASTICINITSEARCHH_MSGEARGS    "Wrong number of arguments"
#define STOCHASTICINITSEARCHH_MSGEARG0    "Wrong value for argv[0]"
#define STOCHASTICINITSEARCHH_MSGENSEG    "Number of data segments is zero or negative"
#define STOCHASTICINITSEARCHH_MSGENPTS    "Number of points in a data segment is zero or negative"
#define STOCHASTICINITSEARCHH_MSGENFRQ    "Number of frequencies is zero or negative"
#define STOCHASTICINITSEARCHH_MSGEF0      "Start frequency is negative"
#define STOCHASTICINITSEARCHH_MSGEDELF    "Frequency spacing is zero or negative"
#define STOCHASTICINITSEARCHH_MSGESTID    "Detector Site ID out of range"
#define STOCHASTICINITSEARCHH_MSGEDTSZ    "Nonstandard size of COMPLEX8 and/or UCHAR" 
#define STOCHASTICINITSEARCHH_MSGEFRES    "Frequency sampling for output finer than for optimal filter"
#define STOCHASTICINITSEARCHH_MSGEFRNG    "Frequency range for output not fully
contained in  optimal filter"
/******** </lalErrTable> ********/

/*
 * libstochasticbar.so should be invoked with the following 
 * arguments to wrapperAPI
 *
 * argv[0]  = "-filterparams"
 * argv[1]  = numSegments       Number of data segments
 * argv[2]  = numPoints         Number of points in a data segment
 * argv[3]  = numFrequencies    Number of frequencies in optimal filter
 * argv[4]  = f0                Start frequency for optimal filter
 * argv[5]  = deltaF            Frequency spacing for optimal filter
 * argv[6]  = outputNumFreq     Number of frequencies in CC spectrum
 * argv[7]  = outputF0          Start frequency for CC spectrum
 * argv[8]  = outputDeltaF      Frequency spacing for CC spectrum
 * argv[9]  = dmro1name         Name of first data channel
 * argv[10]  = dmro2name         Name of second data channel
 * argv[11]  = response1name     Name of first response fcn
 * argv[12]  = response2name     Name of second response fcn
 * argv[13] = epochsMatch       Check that time streams are simultaneous
 * argv[14] = siteID1           Index of first detector in lalCachedDetectors
 * argv[15] = siteID2           Index of second detector in lalCachedDetectors
 * argv[16] = spectrum1name     Name of first PSD
 * argv[17] = spectrum2name     Name of second PSD
 * argv[18] = allegroLatitude	Latitude of ALLEGRO
 * argv[19] = allegroLongitude	Longitude of ALLEGRO
 * argv[20] = allegroAzimuth	Azimuth of ALLEGRO
 *
 */

#define STOCHASTICINITSEARCHH_NPARAMS 21

#ifdef  __cplusplus
}
#endif /* C++ protection */

#endif /* _STOCHASTICINITSEARCH_H  */


/******************************* <lalLaTeX file="StochasticBarInitSearchH">
  
\vfill{\footnotesize\input{StochasticBarInitSearchHV}}

\newpage\input{StochasticBarInitSearchC}

*************************************************** </lalLaTeX> */
