/************************** <lalVerbatim file="StochasticConditionDataCV">
Author: Anderson, W.G., Romano, J.D., and Whelan, J.T.
$Id: StochasticConditionData.c,v 1.17 2001/08/27 18:23:45 whelan Exp $
**************************************************** </lalVerbatim> */

/**************************** <lalLaTeX file="StochasticConditionDataC">
\subsection{Module \texttt{StochasticConditionData.c}}
\label{s:StochasticConditionData.c}

The LAL-wrapper interface function for ...

\subsection*{Description}

\subsection*{Algorithms}
  
\subsection*{Uses}

\subsection*{Notes}

\vfill{\footnotesize\input{StochasticConditionDataCV}}

*************************************************** </lalLaTeX> */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/DetectorSite.h>
#include <lal/CoarseGrainFrequencySeries.h>
#include "Stochastic.h"
#include "StochasticConditionData.h"

NRCSID (STOCHASTICCONDITIONDATAC, "$Id: StochasticConditionData.c,v 1.17 2001/08/27 18:23:45 whelan Exp $");


static void CastToReal4 ( LALStatus *status, REAL4 *output,
			  const multiDimData *input,
			  const UINT4 offset, const UINT4 length );

static void CastToComplex8 ( LALStatus *status, COMPLEX8 *output,
			     const multiDimData *input,
			     const UINT4 offset, const UINT4 length );

void
LALConditionData (
    LALStatus*          status,
    LALSearchInput*     inout,
    void*               searchParams,
    LALMPIParams          *mpiParams
)
{
    INT4                            i;

    /* :NOTE: Will be put back when detector info can be read from frames */
    // INT4                            Geom1  = 0;
    // INT4                            Geom2  = 0;
    INT4                            spec1Index      = -1;
    INT4                            spec2Index      = -1;
    INT4                            resp1Index      = -1;
    INT4                            resp2Index      = -1;
    INT4                            dmro1Index      = -1;
    INT4                            dmro2Index      = -1;

    /* :NOTE: Will be put back when detector info can be read from frames */
    // LALFrDetector*                  FrDetector1;
    // LALFrDetector*                  FrDetector2;

    REAL4FrequencySeries           *spec1, *spec2;
    REAL4FrequencySeries            spec1Raw, spec2Raw;
    REAL4Vector                    *spec1TwoSidedData, *spec2TwoSidedData;
    COMPLEX8FrequencySeries        *resp1, *resp2;
    COMPLEX8FrequencySeries         resp1Raw, resp2Raw;
    COMPLEX8Vector                 *resp1TwoSidedData, *resp2TwoSidedData;


    INT4                            offset;

    FrequencySamplingParams         freqParams;
    
    REAL4FrequencySeries            overlap;
    LALDetectorPair                 detectorPair;
    OverlapReductionFunctionParameters overlapParams;

    REAL4FrequencySeries            omegaGW;
    StochasticOmegaGWParameters     omegaGWParams;

    REAL4FrequencySeries            unWhitenedInvNoise1;
    REAL4FrequencySeries            unWhitenedInvNoise2;
    COMPLEX8FrequencySeries         halfWhitenedInvNoise1;
    COMPLEX8FrequencySeries         halfWhitenedInvNoise2;
    
    StochasticInverseNoiseInput     invNoiseInput1;
    StochasticInverseNoiseInput     invNoiseInput2;
    StochasticInverseNoiseOutput    invNoiseOutput1;
    StochasticInverseNoiseOutput    invNoiseOutput2;
    
    StochasticOptimalFilterInput    optimalInput;
    StochasticOptimalFilterParameters    optimalParams;
    
    StochasticDataSegmentVector*    dataSegVec      = NULL;
    StochasticDataSegment*          dummySegment    = NULL;
    StochasticSearchParams*         params          = NULL;

    INT8                            epochNanoSeconds;

    INITSTATUS (status, "LALConditionData", STOCHASTICCONDITIONDATAC);
    ATTATCHSTATUSPTR (status);

    /* 
     *
     * check and cast searchParams 
     *
     */
    
    ASSERT(searchParams,status,STOCHASTICCONDITIONDATAH_ENULL, 
              STOCHASTICCONDITIONDATAH_MSGENULL);
    
    params = (StochasticSearchParams *) searchParams;

    /* 
     *
     * make sure conditioning and analysis of data is done on search master 
     *
     */
    
    if (!(params->searchMaster))
    {
        ABORT(status, 
              STOCHASTICCONDITIONDATAH_EMSTR, 
              STOCHASTICCONDITIONDATAH_MSGEMSTR);
    }

    /*
     *
     *  check the input 
     *
     */

    /* check that inout data exists */
    ASSERT (inout, 
            status,
            STOCHASTICCONDITIONDATAH_ENULL, 
            STOCHASTICCONDITIONDATAH_MSGENULL);
            
    /* check that number of sequences is correct */ 
    /* :NOTE: once the detector data are included this will be 8 */
    if (inout->numberSequences != 6)
    {
        ABORT(status, 
              STOCHASTICCONDITIONDATAH_ENSEQ, 
              STOCHASTICCONDITIONDATAH_MSGENSEQ);
    }
       
    /* check that multiDimData sequence data exists */
    if (!(inout->sequences))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_ENULL, 
               STOCHASTICCONDITIONDATAH_MSGENULL);
    }
       
    
    /*
     *  
     * identify multiDimData sequences by their names, and check that 
     * the data has the correct type, dimensions, and length 
     *
     */
    
    for (i = 0; i < inout->numberSequences; ++i)
    {
      //      /* look for detector 1 info */
      //      if (!strncmp (inout->sequences[i].name, 
      //		    STOCHASTICINPUTNAME_DETECTOR1, 
      //		    strlen (STOCHASTICINPUTNAME_DETECTOR1))) 
      //      {
      //	Geom1 = i;
      //	/* :TODO: need to have pointer to detector info */
      //	  /* need info from MPIteam/Kent first. WGA */
      //      }
      //      /* look for data from Livingston */
      //      else if (!strncmp (inout->sequences[i].name, 
      //			 STOCHASTICINPUTNAME_DETECTOR2, 
      //			 strlen (STOCHASTICINPUTNAME_DETECTOR2))) 
      //      {
      //	Geom2 = i;
      //	/* :TODO: need to have pointer to detector info */
      //	/* need info from MPIteam/Kent first. WGA */
      //      }
      /*   else */ if (strstr (inout->sequences[i].name, 
			       params->initParams->spectrum1name))
		       //      strlen (params->initParams->spectrum1name))) 
      {
	spec1Index = i;

	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}

	//	/* check length */
	//	if (inout->sequences[i].dimensions[0] 
	//	    != params->initParams->numFrequencies)
	//	{
	//	  ABORT (status, 
	//		 STOCHASTICCONDITIONDATAH_ESIZE, 
	//		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	//	}
            
      }
      else if (strstr (inout->sequences[i].name, 
		       params->initParams->spectrum2name))
      {
	spec2Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	//	/* check length */
	//	if (inout->sequences[i].dimensions[0] 
	//	    != params->initParams->numFrequencies)
	//	{
	//	  ABORT (status, 
	//		 STOCHASTICCONDITIONDATAH_ESIZE, 
	//		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	//	}
	
      }
      else if (strstr (inout->sequences[i].name, 
		       params->initParams->response1name))
      {
	resp1Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	//	/* check length */
	//	if (inout->sequences[i].dimensions[0] 
	//	    != params->initParams->numFrequencies)
	//	{
	//	  ABORT (status, 
	//		 STOCHASTICCONDITIONDATAH_ESIZE, 
	//		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	//	}
	
      }
      else if (strstr (inout->sequences[i].name, 
		       params->initParams->response2name))
      {
	resp2Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	//	/* check length */
	//	if (inout->sequences[i].dimensions[0] 
	//	    != params->initParams->numFrequencies)
	//	{
	//	  ABORT (status, 
	//		 STOCHASTICCONDITIONDATAH_ESIZE, 
	//		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	//	}
	
      }
      else if (strstr (inout->sequences[i].name,
		       params->initParams->dmro1name))
      {
	dmro1Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	/* check length */
	if (inout->sequences[i].dimensions[0] 
	    != params->initParams->numSegments *
	    params->initParams->numPoints)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ESIZE, 
		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	}
      }
      else if (strstr (inout->sequences[i].name,
		       params->initParams->dmro2name))
      {
	dmro2Index = i;
	
	/* check dimensions */
	if (inout->sequences[i].numberDimensions != 1)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_EDIMS, 
		 STOCHASTICCONDITIONDATAH_MSGEDIMS);
	}
	
	/* check length */
	if (inout->sequences[i].dimensions[0] 
	    != params->initParams->numSegments *
	    params->initParams->numPoints)
	{
	  ABORT (status, 
		 STOCHASTICCONDITIONDATAH_ESIZE, 
		 STOCHASTICCONDITIONDATAH_MSGESIZE);
	}	
      }
      else
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_ENAME, 
	       STOCHASTICCONDITIONDATAH_MSGENAME); 
      }
    }
    
    /* 
     *
     * allocate memory for non-trivial parameter fields 
     *
     */
   
    /* detector1 */ 
    /*
    FrDetector1 = (LALFrDetector *) LALMalloc (sizeof (LALFrDetector));
    if (!(FrDetector1))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    */

    /* detector2 */
    /* 
    FrDetector2 = (LALFrDetector *) 
        LALMalloc (sizeof (LALFrDetector));
    if (!(FrDetector2))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    */

    /* spec1 */
    spec1 = (REAL4FrequencySeries *) 
        LALMalloc (sizeof (REAL4FrequencySeries));
    if (!(spec1))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    
    spec1->data = NULL;
    LALCreateVector (status->statusPtr,
                     &(spec1->data),
                     params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
                    
    /* spec2 */    
    spec2 = (REAL4FrequencySeries *) 
        LALMalloc (sizeof (REAL4FrequencySeries));
    if (!(spec2))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    
    spec2->data = NULL;
    LALCreateVector (status->statusPtr,
                     &(spec2->data),
                     params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    /* resp1 */    
    resp1 = (COMPLEX8FrequencySeries *) 
        LALMalloc (sizeof (COMPLEX8FrequencySeries));
    if (!(resp1))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
   
    resp1->data = NULL;
    LALCCreateVector (status->statusPtr,
                      &(resp1->data),
                      params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    /* resp2 */ 
    resp2 = (COMPLEX8FrequencySeries *) 
        LALMalloc (sizeof (COMPLEX8FrequencySeries));
    if (!(resp2))
    {
        ABORT (status, 
               STOCHASTICCONDITIONDATAH_EALOC, 
               STOCHASTICCONDITIONDATAH_MSGEALOC);
    }
    
    resp2->data = NULL;
    LALCCreateVector (status->statusPtr,
                      &(resp2->data),
                      params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);


    /*
     *
     * translate detector geometry InPut to LALFrDetector structures
     *
     */
    /* :TODO: change code when detector info in frames is sorted out. */
    
    /* Hard code nonsense values (except for name) for detector 1 -
     * LALCreateDetector() should ignore and used cached values 
     * Actually, We'll get them from the cache for now
     */
    /*
    strcpy(FrDetector1->name,"LIGO Hanford Observatory");
    FrDetector1->vertexLongitudeDegrees = 0;
    FrDetector1->vertexLatitudeDegrees = 0;
    FrDetector1->vertexElevation = 0;
    FrDetector1->xArmAltitudeRadians = 0;
    FrDetector1->xArmAzimuthRadians = 0;
    FrDetector1->yArmAltitudeRadians = 0;
    FrDetector1->yArmAzimuthRadians = 0;
    */

    /* Hard code nonsense values (except for name) for detector 2 -
     * LALCreateDetector() should ignore and used cached values 
     * Actually, We'll get them from the cache for now
     */
    /*
    strcpy(FrDetector2->name,"LIGO Livingston Observatory");
    FrDetector2->vertexLongitudeDegrees = 0;
    FrDetector2->vertexLatitudeDegrees = 0;
    FrDetector2->vertexElevation = 0;
    FrDetector2->xArmAltitudeRadians = 0;
    FrDetector2->xArmAzimuthRadians = 0;
    FrDetector2->yArmAltitudeRadians = 0;
    FrDetector2->yArmAzimuthRadians = 0;
    */

    /*
     *
     * Fill in frequency sampling parameters from initparams
     *
     */

    freqParams.f0 = params->initParams->f0;
    freqParams.deltaF = params->initParams->deltaF;
    freqParams.length = params->initParams->numFrequencies;

    /* 
     *
     * translate spectrum InPut to REAL4FrequencySeries
     * and coarse grain to desired resolution
     *
     */
    
    /* spec1 */
    spec1TwoSidedData = NULL;
    TRY( LALSCreateVector(status->statusPtr,
			  &spec1TwoSidedData,
			  inout->sequences[spec1Index].dimensions[0]),
	 status );

    CastToReal4 ( status, spec1TwoSidedData->data,
		  &(inout->sequences[spec1Index]),
		  0, spec1TwoSidedData->length );

    /* check if this is a one- or two-sided spectrum */
    if ( inout->sequences[spec1Index].range.dFreq.startFreq >= 0) {
      spec1Raw.data = spec1TwoSidedData;
      spec1Raw.f0 = inout->sequences[spec1Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[spec1Index].range.dFreq.startFreq
		  / inout->sequences[spec1Index].range.dFreq.freqStepSize
		  + 0.5);
      spec1Raw.data = (REAL4Vector *)
	LALMalloc(sizeof(REAL4Vector));

      if (!(spec1Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      spec1Raw.data->length = spec1TwoSidedData->length - offset;
      spec1Raw.data->data = spec1TwoSidedData->data + offset;
      spec1Raw.f0 = 0.0;
    }

    strncpy (spec1Raw.name, 
             params->initParams->spectrum1name, 
             LALNameLength * sizeof(CHAR));

    spec1Raw.deltaF = 
        inout->sequences[spec1Index].range.dFreq.freqStepSize;

    spec1Raw.epoch.gpsSeconds = 
      (inout->sequences[spec1Index].range.dFreq.gpsStartTimeSec);

    spec1Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[spec1Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    spec1Raw.sampleUnits = lalDimensionlessUnit;
    spec1Raw.sampleUnits.unitNumerator[LALUnitIndexADCCount] = 2;
    spec1Raw.sampleUnits.unitNumerator[LALUnitIndexSecond]   = 1;

    TRY( LALSCoarseGrainFrequencySeries(status->statusPtr,
					spec1,
					&spec1Raw,
					&freqParams),
	 status );

    if (spec1Raw.data != spec1TwoSidedData) {
      LALFree(spec1Raw.data);
    }

    TRY( LALSDestroyVector(status->statusPtr,
			   &spec1TwoSidedData),
	 status );    

    /* spec2 */
    spec2TwoSidedData = NULL;
    TRY( LALSCreateVector(status->statusPtr,
			  &spec2TwoSidedData,
			  inout->sequences[spec2Index].dimensions[0]),
	 status );

    CastToReal4 ( status, spec2TwoSidedData->data,
		  &(inout->sequences[spec2Index]),
		  0, spec2TwoSidedData->length );

    /* check if this is a one- or two-sided spectrum */
    if ( inout->sequences[spec2Index].range.dFreq.startFreq >= 0) {
      spec2Raw.data = spec2TwoSidedData;
      spec2Raw.f0 = inout->sequences[spec2Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[spec2Index].range.dFreq.startFreq
		  / inout->sequences[spec2Index].range.dFreq.freqStepSize
		  + 0.5);
      spec2Raw.data = (REAL4Vector *)
	LALMalloc(sizeof(REAL4Vector));

      if (!(spec2Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      spec2Raw.data->length = spec2TwoSidedData->length - offset;
      spec2Raw.data->data = spec2TwoSidedData->data + offset;
      spec2Raw.f0 = 0.0;
    }

    strncpy (spec2Raw.name, 
             params->initParams->spectrum1name, 
             LALNameLength * sizeof(CHAR));

    spec2Raw.deltaF = 
        inout->sequences[spec2Index].range.dFreq.freqStepSize;

    spec2Raw.epoch.gpsSeconds = 
      (inout->sequences[spec2Index].range.dFreq.gpsStartTimeSec);

    spec2Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[spec2Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    spec2Raw.sampleUnits = lalDimensionlessUnit;
    spec2Raw.sampleUnits.unitNumerator[LALUnitIndexADCCount] = 2;
    spec2Raw.sampleUnits.unitNumerator[LALUnitIndexSecond]   = 1;

    TRY( LALSCoarseGrainFrequencySeries(status->statusPtr,
					spec2,
					&spec2Raw,
					&freqParams),
	 status );

    if (spec2Raw.data != spec2TwoSidedData) {
      LALFree(spec2Raw.data);
    }

    TRY( LALSDestroyVector(status->statusPtr,
			   &spec2TwoSidedData),
	 status );    
    
    /*
     *  
     *  translate response InPut to COMPLEX8FrequencySeries
     *  and coarse grain to desired resolution
     *
     */

    /* resp1 */
    resp1TwoSidedData = NULL;
    TRY( LALCCreateVector(status->statusPtr,
			  &resp1TwoSidedData,
			  inout->sequences[resp1Index].dimensions[0]),
	 status );

    CastToComplex8 ( status, resp1TwoSidedData->data,
		     &(inout->sequences[resp1Index]),
		     0, resp1TwoSidedData->length );

    /* need to discard redundant negative frequencies to make
     * LALCCoarseGrainFrequencySeries happy 
     */

    if ( inout->sequences[resp1Index].range.dFreq.startFreq >= 0) {
      resp1Raw.data = resp1TwoSidedData;
      resp1Raw.f0 = inout->sequences[resp1Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[resp1Index].range.dFreq.startFreq
		  / inout->sequences[resp1Index].range.dFreq.freqStepSize
		  + 0.5);
      resp1Raw.data = (COMPLEX8Vector *)
	LALMalloc(sizeof(COMPLEX8Vector));

      if (!(resp1Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      resp1Raw.data->length = resp1TwoSidedData->length - offset;
      resp1Raw.data->data = resp1TwoSidedData->data + offset;
      resp1Raw.f0 = 0.0;
    }

    strncpy (resp1Raw.name, 
             params->initParams->response1name, 
             LALNameLength * sizeof(CHAR));

    resp1Raw.deltaF = 
        inout->sequences[resp1Index].range.dFreq.freqStepSize;

    resp1Raw.epoch.gpsSeconds = 
      (inout->sequences[resp1Index].range.dFreq.gpsStartTimeSec);

    resp1Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[resp1Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    resp1Raw.sampleUnits = lalADCCountUnit;
    resp1Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
    resp1Raw.sampleUnits.powerOfTen = 18;

    TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					resp1,
					&resp1Raw,
					&freqParams),
	 status );

    if (resp1Raw.data != resp1TwoSidedData) {
      LALFree(resp1Raw.data);
    }

    TRY( LALCDestroyVector(status->statusPtr,
			   &resp1TwoSidedData),
	 status );    

    /* resp2 */
    resp2TwoSidedData = NULL;
    TRY( LALCCreateVector(status->statusPtr,
			  &resp2TwoSidedData,
			  inout->sequences[resp2Index].dimensions[0]),
	 status );

    CastToComplex8 ( status, resp2TwoSidedData->data,
		     &(inout->sequences[resp2Index]),
		     0, resp2TwoSidedData->length );

    /* need to discard redundant negative frequencies to make
     * LALCCoarseGrainFrequencySeries happy 
     */

    if ( inout->sequences[resp2Index].range.dFreq.startFreq >= 0) {
      resp2Raw.data = resp2TwoSidedData;
      resp2Raw.f0 = inout->sequences[resp2Index].range.dFreq.startFreq;
    }
    else 
    {
      offset 
	= (INT4) (- inout->sequences[resp2Index].range.dFreq.startFreq
		  / inout->sequences[resp2Index].range.dFreq.freqStepSize
		  + 0.5);
      resp2Raw.data = (COMPLEX8Vector *)
	LALMalloc(sizeof(COMPLEX8Vector));

      if (!(resp2Raw.data))
      {
	ABORT (status, 
	       STOCHASTICCONDITIONDATAH_EALOC, 
	       STOCHASTICCONDITIONDATAH_MSGEALOC);
      }

      resp2Raw.data->length = resp2TwoSidedData->length - offset;
      resp2Raw.data->data = resp2TwoSidedData->data + offset;
      resp2Raw.f0 = 0.0;
    }

    strncpy (resp2Raw.name, 
             params->initParams->response2name, 
             LALNameLength * sizeof(CHAR));

    resp2Raw.deltaF = 
        inout->sequences[resp2Index].range.dFreq.freqStepSize;

    resp2Raw.epoch.gpsSeconds = 
      (inout->sequences[resp2Index].range.dFreq.gpsStartTimeSec);

    resp2Raw.epoch.gpsNanoSeconds = 
      (inout->sequences[resp2Index].range.dFreq.gpsStartTimeNan);

    /* really should get this from the ILWD */
    resp2Raw.sampleUnits = lalADCCountUnit;
    resp2Raw.sampleUnits.unitNumerator[LALUnitIndexStrain]   = -1;
    resp2Raw.sampleUnits.powerOfTen = 18;

    TRY( LALCCoarseGrainFrequencySeries(status->statusPtr,
					resp2,
					&resp2Raw,
					&freqParams),
	 status );

    if (resp2Raw.data != resp2TwoSidedData) {
      LALFree(resp2Raw.data);
    }

    TRY( LALCDestroyVector(status->statusPtr,
			   &resp2TwoSidedData),
	 status );    

    /* 
     *
     * translate ifodmro InPut to DataSegment 
     *
     */ 
    
    dataSegVec = params->stochasticSegVec;
    
    for (i = 0; i < dataSegVec->length; ++i)
    {
        /* point to current segment */
        dummySegment = dataSegVec->data + i;

        /* 
         * 
         * copy the ifodmro data 
         * 
         */
        
        /* data1 */
	CastToReal4 ( status, dummySegment->data1->data->data,
		      &(inout->sequences[dmro1Index]),
		      i * params->initParams->numPoints,
		      params->initParams->numPoints );

    /*
	memcpy (dummySegment->data1->data->data,
		dummydmro1R4,
		params->initParams->numPoints * sizeof (REAL4));
    */
        
        dummySegment->data1->data->length = params->initParams->numPoints;

        strncpy (dummySegment->data1->name,
		 inout->sequences[dmro1Index].name,
                 LALNameLength * sizeof(CHAR));
        
        dummySegment->data1->deltaT = 
            inout->sequences[dmro1Index].range.dTime.timeStepSize;
        
	epochNanoSeconds = 1000000000L
	  * (INT8) inout->sequences[dmro1Index].range.dTime.startSec;
	epochNanoSeconds 
	  += (INT8) inout->sequences[dmro1Index].range.dTime.startNan;
	// printf("%d\n",(INT4) epochNanoSeconds);
	epochNanoSeconds
	  += (INT8) ( 1e9 * (REAL8) i 
		      * (REAL8) (params->initParams->numPoints) 
		      * dummySegment->data1->deltaT );

	/* printf("%d %d %d %f %f\n",(INT4) epochNanoSeconds, i, 
	       params->initParams->numPoints, dummySegment->data1->deltaT,
	       (REAL8) i * (REAL8) (params->initParams->numPoints) 
	       * dummySegment->data1->deltaT); */

        dummySegment->data1->epoch.gpsSeconds
	  = (INT4) (epochNanoSeconds / 1000000000L);
        dummySegment->data1->epoch.gpsNanoSeconds
	  = (INT4) (epochNanoSeconds % 1000000000L);

	/* printf ("%d %d %d\n", i, dummySegment->data1->epoch.gpsSeconds,
	   dummySegment->data1->epoch.gpsNanoSeconds); */

        dummySegment->data1->f0 = 0.0;

	/* really should get this from the ILWD */
	dummySegment->data1->sampleUnits = lalADCCountUnit;

        /* data2 */
	CastToReal4 ( status, dummySegment->data2->data->data,
		      &(inout->sequences[dmro2Index]),
		      i * params->initParams->numPoints,
		      params->initParams->numPoints );
	/*
	memcpy (dummySegment->data2->data->data,
		dummydmro2R4,
		params->initParams->numPoints * sizeof (REAL4));
	*/

        dummySegment->data2->data->length = params->initParams->numPoints;

        strncpy (dummySegment->data2->name,
		 inout->sequences[dmro2Index].name,
                 LALNameLength * sizeof(CHAR));
        
        dummySegment->data2->deltaT = 
            inout->sequences[dmro2Index].range.dTime.timeStepSize;
        
	epochNanoSeconds = 1000000000L
	  * (INT8) inout->sequences[dmro2Index].range.dTime.startSec;
	epochNanoSeconds 
	  += (INT8) inout->sequences[dmro2Index].range.dTime.startNan;
	epochNanoSeconds
	  += (INT8) ( 1e9 * (REAL8) i 
		      * (REAL8) (params->initParams->numPoints) 
		      * dummySegment->data2->deltaT );

        dummySegment->data2->epoch.gpsSeconds
	  = (INT4) (epochNanoSeconds / 1000000000L);
        dummySegment->data2->epoch.gpsNanoSeconds
	  = (INT4) (epochNanoSeconds % 1000000000L);

        dummySegment->data2->f0 = 0.0;

	/* really should get this from the ILWD */
	dummySegment->data2->sampleUnits = lalADCCountUnit;
        
        /* increment pointers to data for next pass through the loop */
	//  dummydmro1R4 += (params->initParams->numPoints);
        //  dummydmro2R4 += (params->initParams->numPoints);

    }
    
    
    /* 
     *
     * calculate overlap reduction function 
     *
     */

    /* allocate memory for frequency series */
    overlap.data = NULL;
    LALCreateVector (status->statusPtr,
                     &(overlap.data),
                     params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    /* fill in OverlapReductionFunctionParameters structure */
    overlapParams.length = params->initParams->numFrequencies;
    overlapParams.f0     = params->initParams->f0;
    overlapParams.deltaF = params->initParams->deltaF;
    
    /* use LALCreateDetector to get the detectorPair fields */
    /*    LALCreateDetector( status, &(detectorPair.detectorOne), FrDetector1, 
          LALDETECTORTYPE_IFODIFF);
    LALCreateDetector( status, &(detectorPair.detectorOne), FrDetector2, 
          LALDETECTORTYPE_IFODIFF);
    */

    /* We'll go back to that once we can get the detector info from
     *  outside; for now, pull it out of the cache */

    detectorPair.detectorOne 
      = lalCachedDetectors[params->initParams->siteID1];
    detectorPair.detectorTwo 
      = lalCachedDetectors[params->initParams->siteID2];
    
    /* calculate overlap reduction function values */
    LALOverlapReductionFunction (status->statusPtr,
                                 &overlap,
                                 &detectorPair,
                                 &overlapParams);
    CHECKSTATUSPTR (status);
   
    
    /*
     *
     * set omega_GW = const 
     *
     */
    
    /* allocate memory for frequency series */
    omegaGW.data = NULL;
    LALCreateVector (status->statusPtr,
                     &(omegaGW.data),
                     params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
     
    /* fill in StochasticOmegaGWParameters structure */
    omegaGWParams.alpha    = 0;
    omegaGWParams.fRef     = params->initParams->f0
      + params->initParams->deltaF;
    omegaGWParams.omegaRef = 1;
    omegaGWParams.length   = params->initParams->numFrequencies;
    omegaGWParams.f0       = params->initParams->f0;
    omegaGWParams.deltaF   = params->initParams->deltaF;
    // printf("Calling OmegaGW\n");
    /* calculate omegaGW */
    LALStochasticOmegaGW (status->statusPtr,
                          &omegaGW,
                          &omegaGWParams);
    CHECKSTATUSPTR (status);
    // printf("Returned from OmegaGW\n");
   
    
    /* 
     *
     * calculate inverse noise psds 
     *
     */

    /* spec1 */
    /* allocate memory for StochasticInverseNoiseOutput fields */
    unWhitenedInvNoise1.data = NULL;
    LALCreateVector (status->statusPtr,
                     &(unWhitenedInvNoise1.data),
                     params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    halfWhitenedInvNoise1.data = NULL;
    LALCCreateVector (status->statusPtr,
                      &(halfWhitenedInvNoise1.data),
                      params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    /* fill in StochasticInverseNoiseInput fields */
    invNoiseInput1.whitenedNoisePSD = spec1;
    invNoiseInput1.whiteningFilter  = resp1;
    
    /* fill in StochasticInverseNoiseOutput fields */
    invNoiseOutput1.unWhitenedInverseNoisePSD   = &unWhitenedInvNoise1;
    invNoiseOutput1.halfWhitenedInverseNoisePSD = &halfWhitenedInvNoise1;

    /* calculate inverse noise outputs */
    LALStochasticInverseNoise (status->statusPtr,
                               &invNoiseOutput1,
                               &invNoiseInput1);
    CHECKSTATUSPTR (status);
     
    /* spec2 */
    /* allocate memory for StochasticInverseNoiseOutput fields */
    unWhitenedInvNoise2.data = NULL;
    LALCreateVector (status->statusPtr,
                     &(unWhitenedInvNoise2.data),
                     params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    halfWhitenedInvNoise2.data = NULL;
    LALCCreateVector (status->statusPtr,
                      &(halfWhitenedInvNoise2.data),
                      params->initParams->numFrequencies);
    CHECKSTATUSPTR (status);
    
    /* fill in StochasticInverseNoiseInput fields */
    invNoiseInput2.whitenedNoisePSD = spec2;
    invNoiseInput2.whiteningFilter  = resp2;
    
    /* fill in StochasticInverseNoiseOutput fields */
    invNoiseOutput2.unWhitenedInverseNoisePSD   = &unWhitenedInvNoise2;
    invNoiseOutput2.halfWhitenedInverseNoisePSD = &halfWhitenedInvNoise2;

    /* calculate inverse noise outputs */
    LALStochasticInverseNoise (status->statusPtr,
                               &invNoiseOutput2,
                               &invNoiseInput2);
    CHECKSTATUSPTR (status);

    
    /*
     *
     * calculate optimal filter 
     *
     */
    
    /* fill in StochasticOptimalFilterInput fields */
    optimalInput.overlapReductionFunction = &overlap;
    optimalInput.omegaGW = &omegaGW;
    optimalInput.halfWhitenedInverseNoisePSD1 = &halfWhitenedInvNoise1;
    optimalInput.halfWhitenedInverseNoisePSD2 = &halfWhitenedInvNoise2;
    optimalInput.unWhitenedInverseNoisePSD1 = &unWhitenedInvNoise1;
    optimalInput.unWhitenedInverseNoisePSD2 = &unWhitenedInvNoise2;

    optimalParams.fRef = omegaGWParams.fRef;
    optimalParams.heterodyned = STOCHASTICH_FALSE ;

    /* calculate optimal filter values */

    LALStochasticOptimalFilter (status->statusPtr,
                                params->optimalFilter,
                                &optimalInput,
                                &optimalParams);
    CHECKSTATUSPTR (status);
 
    
    /*
     *
     * clean up
     *
     */
    
    LALCDestroyVector (status->statusPtr,
                       &(halfWhitenedInvNoise2.data));
    CHECKSTATUSPTR (status);
    
    LALDestroyVector (status->statusPtr,
                      &(unWhitenedInvNoise2.data));
    CHECKSTATUSPTR (status);
    
    LALCDestroyVector (status->statusPtr,
                       &(halfWhitenedInvNoise1.data));
    CHECKSTATUSPTR (status);
    
    LALDestroyVector (status->statusPtr,
                      &(unWhitenedInvNoise1.data));
    CHECKSTATUSPTR (status);
    
    LALDestroyVector (status->statusPtr,
                      &(omegaGW.data));
    CHECKSTATUSPTR (status);
    
    LALDestroyVector (status->statusPtr,
                      &(overlap.data));
    CHECKSTATUSPTR (status);

    TRY( LALDestroyVector (status->statusPtr, &(spec1->data)),
	 status);

    LALFree(spec1);

    TRY( LALDestroyVector (status->statusPtr, &(spec2->data)),
	 status);

    LALFree(spec2);

    TRY( LALCDestroyVector (status->statusPtr, &(resp1->data)),
	 status);

    LALFree(resp1);

    TRY( LALCDestroyVector (status->statusPtr, &(resp2->data)),
	 status);

    LALFree(resp2);
    
    /* normal exit */
    DETATCHSTATUSPTR (status);
    RETURN (status);
}

static void CastToReal4 ( LALStatus *status, REAL4 *output,
			  const multiDimData *input,
			  const UINT4 offset, const UINT4 length )
{
  REAL4     *sPtr, *sStopPtr;
  REAL8     *dPtr;
  INT2      *i2Ptr;
  UINT2     *u2Ptr;
  INT4      *i4Ptr;
  UINT4     *u4Ptr;
  INT8      *i8Ptr;
  UINT8     *u8Ptr;
  COMPLEX8  *cPtr;
  COMPLEX16 *zPtr;

  /* printf ("%d %d\n", input->type, real_4); */
  switch (input->type)
  {
    case real_4:
      memcpy ( output, input->data.real4 + offset,
	       length * sizeof(REAL4) );
      break;
    case real_8:
      sStopPtr = output + length;
      for ( sPtr = output, dPtr = input->data.real8 + offset;
	    sPtr < sStopPtr; ++sPtr, ++dPtr )
      {
	*sPtr = (REAL4) *dPtr;
      }
    break;
    case int_2s:
      sStopPtr = output + length;
      for ( sPtr = output, i2Ptr = input->data.int2s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i2Ptr )
      {
	*sPtr = (REAL4) *i2Ptr;
      }
    break;
    case int_2u:
      sStopPtr = output + length;
      for ( sPtr = output, u2Ptr = input->data.int2u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u2Ptr )
      {
	*sPtr = (REAL4) *u2Ptr;
      }
    break;
    case int_4u:
      sStopPtr = output + length;
      for ( sPtr = output, u4Ptr = input->data.int4u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u4Ptr )
      {
	*sPtr = (REAL4) *u4Ptr;
      }
    break;
    case int_4s:
      sStopPtr = output + length;
      for ( sPtr = output, i4Ptr = input->data.int4s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i4Ptr )
      {
	*sPtr = (REAL4) *i4Ptr;
      }
    break;
    case int_8u:
      sStopPtr = output + length;
      for ( sPtr = output, u8Ptr = input->data.int8u + offset;
	    sPtr < sStopPtr; ++sPtr, ++u8Ptr )
      {
	*sPtr = (REAL4) *u8Ptr;
      }
    break;
    case int_8s:
      sStopPtr = output + length;
      for ( sPtr = output, i8Ptr = input->data.int8s + offset;
	    sPtr < sStopPtr; ++sPtr, ++i8Ptr )
      {
	*sPtr = (REAL4) *i8Ptr;
      }
    break;
    case complex_8:
      sStopPtr = output + length;
      for ( sPtr = output, cPtr = input->data.complex8 + offset;
	    sPtr < sStopPtr; ++sPtr, ++cPtr )
      {
	*sPtr = (REAL4) cPtr->re;
      }
    break;
    case complex_16:
      sStopPtr = output + length;
      for ( sPtr = output, zPtr = input->data.complex16 + offset;
	    sPtr < sStopPtr; ++sPtr, ++zPtr )
      {
	*sPtr = (REAL4) zPtr->re;
      }
    default:
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_ETYPE, 
	     STOCHASTICCONDITIONDATAH_MSGETYPE);
  }
}

static void CastToComplex8 ( LALStatus *status, COMPLEX8 *output,
			     const multiDimData *input,
			     const UINT4 offset, const UINT4 length )
{
  REAL4     *sPtr;
  REAL8     *dPtr;
  INT2      *i2Ptr;
  UINT2     *u2Ptr;
  INT4      *i4Ptr;
  UINT4     *u4Ptr;
  INT8      *i8Ptr;
  UINT8     *u8Ptr;
  COMPLEX8  *cPtr, *cStopPtr;
  COMPLEX16 *zPtr;

  /* printf ("%d %d\n", input->type, complex_8); */
  switch (input->type)
  {
    case complex_8:
      memcpy ( output, input->data.complex8 + offset,
	       length * sizeof(COMPLEX8) );
    break;
    case complex_16:
      cStopPtr = output + length;
      for ( cPtr = output, zPtr = input->data.complex16 + offset;
	    cPtr < cStopPtr; ++cPtr, ++zPtr )
      {
	cPtr->re = (REAL4) zPtr->re;
	cPtr->im = (REAL4) zPtr->im;
      }
    break;
    case real_4:
      cStopPtr = output + length;
      for ( cPtr = output, sPtr = input->data.real4 + offset;
	    cPtr < cStopPtr; ++cPtr, ++dPtr )
      {
	cPtr->re = *sPtr;
	cPtr->im = 0.0;
      }
    break;
    case real_8:
      cStopPtr = output + length;
      for ( cPtr = output, dPtr = input->data.real8 + offset;
	    cPtr < cStopPtr; ++cPtr, ++dPtr )
      {
	cPtr->re = (REAL4) *dPtr;
	cPtr->im = 0.0;
      }
    break;
    case int_2s:
      cStopPtr = output + length;
      for ( cPtr = output, i2Ptr = input->data.int2s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i2Ptr )
      {
	cPtr->re = (REAL4) *i2Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_2u:
      cStopPtr = output + length;
      for ( cPtr = output, u2Ptr = input->data.int2u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u2Ptr )
      {
	cPtr->re = (REAL4) *u2Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_4u:
      cStopPtr = output + length;
      for ( cPtr = output, u4Ptr = input->data.int4u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u4Ptr )
      {
	cPtr->re = (REAL4) *u4Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_4s:
      cStopPtr = output + length;
      for ( cPtr = output, i4Ptr = input->data.int4s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i4Ptr )
      {
	cPtr->re = (REAL4) *i4Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_8u:
      cStopPtr = output + length;
      for ( cPtr = output, u8Ptr = input->data.int8u + offset;
	    cPtr < cStopPtr; ++cPtr, ++u8Ptr )
      {
	cPtr->re = (REAL4) *u8Ptr;
	cPtr->im = 0.0;
      }
    break;
    case int_8s:
      cStopPtr = output + length;
      for ( cPtr = output, i8Ptr = input->data.int8s + offset;
	    cPtr < cStopPtr; ++cPtr, ++i8Ptr )
      {
	cPtr->re = (REAL4) *i8Ptr;
	cPtr->im = 0.0;
      }
    break;
    default:
      ABORT (status, 
	     STOCHASTICCONDITIONDATAH_ETYPE, 
	     STOCHASTICCONDITIONDATAH_MSGETYPE);
  }
}
